# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'GetTimeResult',
    'AwaitableGetTimeResult',
    'get_time',
    'get_time_output',
]

@pulumi.output_type
class GetTimeResult:
    """
    A collection of values returned by getTime.
    """
    def __init__(__self__, id=None, local_time=None, node_name=None, time_zone=None, utc_time=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if local_time and not isinstance(local_time, str):
            raise TypeError("Expected argument 'local_time' to be a str")
        pulumi.set(__self__, "local_time", local_time)
        if node_name and not isinstance(node_name, str):
            raise TypeError("Expected argument 'node_name' to be a str")
        pulumi.set(__self__, "node_name", node_name)
        if time_zone and not isinstance(time_zone, str):
            raise TypeError("Expected argument 'time_zone' to be a str")
        pulumi.set(__self__, "time_zone", time_zone)
        if utc_time and not isinstance(utc_time, str):
            raise TypeError("Expected argument 'utc_time' to be a str")
        pulumi.set(__self__, "utc_time", utc_time)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="localTime")
    def local_time(self) -> str:
        """
        The node's local time.
        """
        return pulumi.get(self, "local_time")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> str:
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter(name="timeZone")
    def time_zone(self) -> str:
        """
        The node's time zone.
        """
        return pulumi.get(self, "time_zone")

    @property
    @pulumi.getter(name="utcTime")
    def utc_time(self) -> str:
        """
        The node's local time formatted as UTC.
        """
        return pulumi.get(self, "utc_time")


class AwaitableGetTimeResult(GetTimeResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetTimeResult(
            id=self.id,
            local_time=self.local_time,
            node_name=self.node_name,
            time_zone=self.time_zone,
            utc_time=self.utc_time)


def get_time(node_name: Optional[str] = None,
             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetTimeResult:
    """
    Retrieves the current time for a specific node.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    first_node_time = proxmoxve.get_time(node_name="first-node")
    ```


    :param str node_name: A node name.
    """
    __args__ = dict()
    __args__['nodeName'] = node_name
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('proxmoxve:index/getTime:getTime', __args__, opts=opts, typ=GetTimeResult).value

    return AwaitableGetTimeResult(
        id=pulumi.get(__ret__, 'id'),
        local_time=pulumi.get(__ret__, 'local_time'),
        node_name=pulumi.get(__ret__, 'node_name'),
        time_zone=pulumi.get(__ret__, 'time_zone'),
        utc_time=pulumi.get(__ret__, 'utc_time'))


@_utilities.lift_output_func(get_time)
def get_time_output(node_name: Optional[pulumi.Input[str]] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[GetTimeResult]:
    """
    Retrieves the current time for a specific node.

    ## Example Usage

    ```python
    import pulumi
    import pulumi_proxmoxve as proxmoxve

    first_node_time = proxmoxve.get_time(node_name="first-node")
    ```


    :param str node_name: A node name.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'HostsEntryArgs',
    'ProviderSshArgs',
    'ProviderSshNodeArgs',
]

@pulumi.input_type
class HostsEntryArgs:
    def __init__(__self__, *,
                 address: pulumi.Input[str],
                 hostnames: pulumi.Input[Sequence[pulumi.Input[str]]]):
        """
        :param pulumi.Input[str] address: The IP address.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] hostnames: The hostnames.
        """
        HostsEntryArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            address=address,
            hostnames=hostnames,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             address: pulumi.Input[str],
             hostnames: pulumi.Input[Sequence[pulumi.Input[str]]],
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):

        _setter("address", address)
        _setter("hostnames", hostnames)

    @property
    @pulumi.getter
    def address(self) -> pulumi.Input[str]:
        """
        The IP address.
        """
        return pulumi.get(self, "address")

    @address.setter
    def address(self, value: pulumi.Input[str]):
        pulumi.set(self, "address", value)

    @property
    @pulumi.getter
    def hostnames(self) -> pulumi.Input[Sequence[pulumi.Input[str]]]:
        """
        The hostnames.
        """
        return pulumi.get(self, "hostnames")

    @hostnames.setter
    def hostnames(self, value: pulumi.Input[Sequence[pulumi.Input[str]]]):
        pulumi.set(self, "hostnames", value)


@pulumi.input_type
class ProviderSshArgs:
    def __init__(__self__, *,
                 agent: Optional[pulumi.Input[bool]] = None,
                 agent_socket: Optional[pulumi.Input[str]] = None,
                 nodes: Optional[pulumi.Input[Sequence[pulumi.Input['ProviderSshNodeArgs']]]] = None,
                 password: Optional[pulumi.Input[str]] = None,
                 username: Optional[pulumi.Input[str]] = None):
        ProviderSshArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            agent=agent,
            agent_socket=agent_socket,
            nodes=nodes,
            password=password,
            username=username,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             agent: Optional[pulumi.Input[bool]] = None,
             agent_socket: Optional[pulumi.Input[str]] = None,
             nodes: Optional[pulumi.Input[Sequence[pulumi.Input['ProviderSshNodeArgs']]]] = None,
             password: Optional[pulumi.Input[str]] = None,
             username: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'agentSocket' in kwargs:
            agent_socket = kwargs['agentSocket']

        if agent is not None:
            _setter("agent", agent)
        if agent_socket is not None:
            _setter("agent_socket", agent_socket)
        if nodes is not None:
            _setter("nodes", nodes)
        if password is not None:
            _setter("password", password)
        if username is not None:
            _setter("username", username)

    @property
    @pulumi.getter
    def agent(self) -> Optional[pulumi.Input[bool]]:
        return pulumi.get(self, "agent")

    @agent.setter
    def agent(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "agent", value)

    @property
    @pulumi.getter(name="agentSocket")
    def agent_socket(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "agent_socket")

    @agent_socket.setter
    def agent_socket(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "agent_socket", value)

    @property
    @pulumi.getter
    def nodes(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['ProviderSshNodeArgs']]]]:
        return pulumi.get(self, "nodes")

    @nodes.setter
    def nodes(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['ProviderSshNodeArgs']]]]):
        pulumi.set(self, "nodes", value)

    @property
    @pulumi.getter
    def password(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "password")

    @password.setter
    def password(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "password", value)

    @property
    @pulumi.getter
    def username(self) -> Optional[pulumi.Input[str]]:
        return pulumi.get(self, "username")

    @username.setter
    def username(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "username", value)


@pulumi.input_type
class ProviderSshNodeArgs:
    def __init__(__self__, *,
                 address: pulumi.Input[str],
                 name: pulumi.Input[str],
                 port: Optional[pulumi.Input[int]] = None):
        ProviderSshNodeArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            address=address,
            name=name,
            port=port,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             address: pulumi.Input[str],
             name: pulumi.Input[str],
             port: Optional[pulumi.Input[int]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):

        _setter("address", address)
        _setter("name", name)
        if port is not None:
            _setter("port", port)

    @property
    @pulumi.getter
    def address(self) -> pulumi.Input[str]:
        return pulumi.get(self, "address")

    @address.setter
    def address(self, value: pulumi.Input[str]):
        pulumi.set(self, "address", value)

    @property
    @pulumi.getter
    def name(self) -> pulumi.Input[str]:
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: pulumi.Input[str]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter
    def port(self) -> Optional[pulumi.Input[int]]:
        return pulumi.get(self, "port")

    @port.setter
    def port(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "port", value)





# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GroupAclArgs',
    'PoolMemberArgs',
    'UserAclArgs',
]

@pulumi.input_type
class GroupAclArgs:
    def __init__(__self__, *,
                 path: pulumi.Input[str],
                 role_id: pulumi.Input[str],
                 propagate: Optional[pulumi.Input[bool]] = None):
        """
        :param pulumi.Input[str] path: The path.
        :param pulumi.Input[str] role_id: The role identifier.
        :param pulumi.Input[bool] propagate: Whether to propagate to child paths.
        """
        GroupAclArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            path=path,
            role_id=role_id,
            propagate=propagate,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             path: pulumi.Input[str],
             role_id: pulumi.Input[str],
             propagate: Optional[pulumi.Input[bool]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'roleId' in kwargs:
            role_id = kwargs['roleId']

        _setter("path", path)
        _setter("role_id", role_id)
        if propagate is not None:
            _setter("propagate", propagate)

    @property
    @pulumi.getter
    def path(self) -> pulumi.Input[str]:
        """
        The path.
        """
        return pulumi.get(self, "path")

    @path.setter
    def path(self, value: pulumi.Input[str]):
        pulumi.set(self, "path", value)

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> pulumi.Input[str]:
        """
        The role identifier.
        """
        return pulumi.get(self, "role_id")

    @role_id.setter
    def role_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "role_id", value)

    @property
    @pulumi.getter
    def propagate(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to propagate to child paths.
        """
        return pulumi.get(self, "propagate")

    @propagate.setter
    def propagate(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "propagate", value)


@pulumi.input_type
class PoolMemberArgs:
    def __init__(__self__, *,
                 datastore_id: Optional[pulumi.Input[str]] = None,
                 id: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 type: Optional[pulumi.Input[str]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None):
        """
        :param pulumi.Input[str] datastore_id: The datastore identifier.
        :param pulumi.Input[str] id: The member identifier.
        :param pulumi.Input[str] node_name: The node name.
        :param pulumi.Input[str] type: The member type.
        :param pulumi.Input[int] vm_id: The virtual machine identifier.
        """
        PoolMemberArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            datastore_id=datastore_id,
            id=id,
            node_name=node_name,
            type=type,
            vm_id=vm_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             datastore_id: Optional[pulumi.Input[str]] = None,
             id: Optional[pulumi.Input[str]] = None,
             node_name: Optional[pulumi.Input[str]] = None,
             type: Optional[pulumi.Input[str]] = None,
             vm_id: Optional[pulumi.Input[int]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'datastoreId' in kwargs:
            datastore_id = kwargs['datastoreId']
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']
        if 'vmId' in kwargs:
            vm_id = kwargs['vmId']

        if datastore_id is not None:
            _setter("datastore_id", datastore_id)
        if id is not None:
            _setter("id", id)
        if node_name is not None:
            _setter("node_name", node_name)
        if type is not None:
            _setter("type", type)
        if vm_id is not None:
            _setter("vm_id", vm_id)

    @property
    @pulumi.getter(name="datastoreId")
    def datastore_id(self) -> Optional[pulumi.Input[str]]:
        """
        The datastore identifier.
        """
        return pulumi.get(self, "datastore_id")

    @datastore_id.setter
    def datastore_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "datastore_id", value)

    @property
    @pulumi.getter
    def id(self) -> Optional[pulumi.Input[str]]:
        """
        The member identifier.
        """
        return pulumi.get(self, "id")

    @id.setter
    def id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "id", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[str]]:
        """
        The node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter
    def type(self) -> Optional[pulumi.Input[str]]:
        """
        The member type.
        """
        return pulumi.get(self, "type")

    @type.setter
    def type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "type", value)

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> Optional[pulumi.Input[int]]:
        """
        The virtual machine identifier.
        """
        return pulumi.get(self, "vm_id")

    @vm_id.setter
    def vm_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "vm_id", value)


@pulumi.input_type
class UserAclArgs:
    def __init__(__self__, *,
                 path: pulumi.Input[str],
                 role_id: pulumi.Input[str],
                 propagate: Optional[pulumi.Input[bool]] = None):
        """
        :param pulumi.Input[str] path: The path.
        :param pulumi.Input[str] role_id: The role identifier.
        :param pulumi.Input[bool] propagate: Whether to propagate to child paths.
        """
        UserAclArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            path=path,
            role_id=role_id,
            propagate=propagate,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             path: pulumi.Input[str],
             role_id: pulumi.Input[str],
             propagate: Optional[pulumi.Input[bool]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'roleId' in kwargs:
            role_id = kwargs['roleId']

        _setter("path", path)
        _setter("role_id", role_id)
        if propagate is not None:
            _setter("propagate", propagate)

    @property
    @pulumi.getter
    def path(self) -> pulumi.Input[str]:
        """
        The path.
        """
        return pulumi.get(self, "path")

    @path.setter
    def path(self, value: pulumi.Input[str]):
        pulumi.set(self, "path", value)

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> pulumi.Input[str]:
        """
        The role identifier.
        """
        return pulumi.get(self, "role_id")

    @role_id.setter
    def role_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "role_id", value)

    @property
    @pulumi.getter
    def propagate(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to propagate to child paths.
        """
        return pulumi.get(self, "propagate")

    @propagate.setter
    def propagate(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "propagate", value)





# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities
from . import outputs
from ._inputs import *

__all__ = ['HostsArgs', 'Hosts']

@pulumi.input_type
class HostsArgs:
    def __init__(__self__, *,
                 entry: pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]],
                 node_name: pulumi.Input[str]):
        """
        The set of arguments for constructing a Hosts resource.
        :param pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]] entry: A host entry (multiple blocks supported).
        :param pulumi.Input[str] node_name: A node name.
        """
        HostsArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            entry=entry,
            node_name=node_name,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             entry: pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]],
             node_name: pulumi.Input[str],
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']

        _setter("entry", entry)
        _setter("node_name", node_name)

    @property
    @pulumi.getter
    def entry(self) -> pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]]:
        """
        A host entry (multiple blocks supported).
        """
        return pulumi.get(self, "entry")

    @entry.setter
    def entry(self, value: pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]]):
        pulumi.set(self, "entry", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Input[str]:
        """
        A node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "node_name", value)


@pulumi.input_type
class _HostsState:
    def __init__(__self__, *,
                 addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 digest: Optional[pulumi.Input[str]] = None,
                 entries: Optional[pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]]] = None,
                 entry: Optional[pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]]] = None,
                 hostnames: Optional[pulumi.Input[Sequence[pulumi.Input[Sequence[pulumi.Input[str]]]]]] = None,
                 node_name: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Hosts resources.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] addresses: The IP addresses.
        :param pulumi.Input[str] digest: The SHA1 digest.
        :param pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]] entries: The host entries (conversion of `addresses` and `hostnames` into
               objects).
        :param pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]] entry: A host entry (multiple blocks supported).
        :param pulumi.Input[Sequence[pulumi.Input[Sequence[pulumi.Input[str]]]]] hostnames: The hostnames.
        :param pulumi.Input[str] node_name: A node name.
        """
        _HostsState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            addresses=addresses,
            digest=digest,
            entries=entries,
            entry=entry,
            hostnames=hostnames,
            node_name=node_name,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
             digest: Optional[pulumi.Input[str]] = None,
             entries: Optional[pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]]] = None,
             entry: Optional[pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]]] = None,
             hostnames: Optional[pulumi.Input[Sequence[pulumi.Input[Sequence[pulumi.Input[str]]]]]] = None,
             node_name: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']

        if addresses is not None:
            _setter("addresses", addresses)
        if digest is not None:
            _setter("digest", digest)
        if entries is not None:
            _setter("entries", entries)
        if entry is not None:
            _setter("entry", entry)
        if hostnames is not None:
            _setter("hostnames", hostnames)
        if node_name is not None:
            _setter("node_name", node_name)

    @property
    @pulumi.getter
    def addresses(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The IP addresses.
        """
        return pulumi.get(self, "addresses")

    @addresses.setter
    def addresses(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "addresses", value)

    @property
    @pulumi.getter
    def digest(self) -> Optional[pulumi.Input[str]]:
        """
        The SHA1 digest.
        """
        return pulumi.get(self, "digest")

    @digest.setter
    def digest(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "digest", value)

    @property
    @pulumi.getter
    def entries(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]]]:
        """
        The host entries (conversion of `addresses` and `hostnames` into
        objects).
        """
        return pulumi.get(self, "entries")

    @entries.setter
    def entries(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]]]):
        pulumi.set(self, "entries", value)

    @property
    @pulumi.getter
    def entry(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]]]:
        """
        A host entry (multiple blocks supported).
        """
        return pulumi.get(self, "entry")

    @entry.setter
    def entry(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['HostsEntryArgs']]]]):
        pulumi.set(self, "entry", value)

    @property
    @pulumi.getter
    def hostnames(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[Sequence[pulumi.Input[str]]]]]]:
        """
        The hostnames.
        """
        return pulumi.get(self, "hostnames")

    @hostnames.setter
    def hostnames(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[Sequence[pulumi.Input[str]]]]]]):
        pulumi.set(self, "hostnames", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[str]]:
        """
        A node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "node_name", value)


class Hosts(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 entry: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['HostsEntryArgs']]]]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages the host entries on a specific node.

        ## Important Notes

        Be careful not to use this resource multiple times for the same node.

        ## Import

        Instances can be imported using the `node_name`, e.g., bash

        ```sh
         $ pulumi import proxmoxve:index/hosts:Hosts first_node_host_entries first-node
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['HostsEntryArgs']]]] entry: A host entry (multiple blocks supported).
        :param pulumi.Input[str] node_name: A node name.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: HostsArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages the host entries on a specific node.

        ## Important Notes

        Be careful not to use this resource multiple times for the same node.

        ## Import

        Instances can be imported using the `node_name`, e.g., bash

        ```sh
         $ pulumi import proxmoxve:index/hosts:Hosts first_node_host_entries first-node
        ```

        :param str resource_name: The name of the resource.
        :param HostsArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(HostsArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            HostsArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 entry: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['HostsEntryArgs']]]]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = HostsArgs.__new__(HostsArgs)

            if entry is None and not opts.urn:
                raise TypeError("Missing required property 'entry'")
            __props__.__dict__["entry"] = entry
            if node_name is None and not opts.urn:
                raise TypeError("Missing required property 'node_name'")
            __props__.__dict__["node_name"] = node_name
            __props__.__dict__["addresses"] = None
            __props__.__dict__["digest"] = None
            __props__.__dict__["entries"] = None
            __props__.__dict__["hostnames"] = None
        super(Hosts, __self__).__init__(
            'proxmoxve:index/hosts:Hosts',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            addresses: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            digest: Optional[pulumi.Input[str]] = None,
            entries: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['HostsEntryArgs']]]]] = None,
            entry: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['HostsEntryArgs']]]]] = None,
            hostnames: Optional[pulumi.Input[Sequence[pulumi.Input[Sequence[pulumi.Input[str]]]]]] = None,
            node_name: Optional[pulumi.Input[str]] = None) -> 'Hosts':
        """
        Get an existing Hosts resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] addresses: The IP addresses.
        :param pulumi.Input[str] digest: The SHA1 digest.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['HostsEntryArgs']]]] entries: The host entries (conversion of `addresses` and `hostnames` into
               objects).
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['HostsEntryArgs']]]] entry: A host entry (multiple blocks supported).
        :param pulumi.Input[Sequence[pulumi.Input[Sequence[pulumi.Input[str]]]]] hostnames: The hostnames.
        :param pulumi.Input[str] node_name: A node name.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _HostsState.__new__(_HostsState)

        __props__.__dict__["addresses"] = addresses
        __props__.__dict__["digest"] = digest
        __props__.__dict__["entries"] = entries
        __props__.__dict__["entry"] = entry
        __props__.__dict__["hostnames"] = hostnames
        __props__.__dict__["node_name"] = node_name
        return Hosts(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def addresses(self) -> pulumi.Output[Sequence[str]]:
        """
        The IP addresses.
        """
        return pulumi.get(self, "addresses")

    @property
    @pulumi.getter
    def digest(self) -> pulumi.Output[str]:
        """
        The SHA1 digest.
        """
        return pulumi.get(self, "digest")

    @property
    @pulumi.getter
    def entries(self) -> pulumi.Output[Sequence['outputs.HostsEntry']]:
        """
        The host entries (conversion of `addresses` and `hostnames` into
        objects).
        """
        return pulumi.get(self, "entries")

    @property
    @pulumi.getter
    def entry(self) -> pulumi.Output[Sequence['outputs.HostsEntry']]:
        """
        A host entry (multiple blocks supported).
        """
        return pulumi.get(self, "entry")

    @property
    @pulumi.getter
    def hostnames(self) -> pulumi.Output[Sequence[Sequence[str]]]:
        """
        The hostnames.
        """
        return pulumi.get(self, "hostnames")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Output[str]:
        """
        A node name.
        """
        return pulumi.get(self, "node_name")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs

__all__ = [
    'Ssh',
    'SshNode',
]

@pulumi.output_type
class Ssh(dict):
    def __init__(__self__, *,
                 agent: Optional[bool] = None,
                 agent_socket: Optional[str] = None,
                 nodes: Optional[Sequence['outputs.SshNode']] = None,
                 password: Optional[str] = None,
                 username: Optional[str] = None):
        Ssh._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            agent=agent,
            agent_socket=agent_socket,
            nodes=nodes,
            password=password,
            username=username,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             agent: Optional[bool] = None,
             agent_socket: Optional[str] = None,
             nodes: Optional[Sequence['outputs.SshNode']] = None,
             password: Optional[str] = None,
             username: Optional[str] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'agentSocket' in kwargs:
            agent_socket = kwargs['agentSocket']

        if agent is not None:
            _setter("agent", agent)
        if agent_socket is not None:
            _setter("agent_socket", agent_socket)
        if nodes is not None:
            _setter("nodes", nodes)
        if password is not None:
            _setter("password", password)
        if username is not None:
            _setter("username", username)

    @property
    @pulumi.getter
    def agent(self) -> Optional[bool]:
        return pulumi.get(self, "agent")

    @property
    @pulumi.getter(name="agentSocket")
    def agent_socket(self) -> Optional[str]:
        return pulumi.get(self, "agent_socket")

    @property
    @pulumi.getter
    def nodes(self) -> Optional[Sequence['outputs.SshNode']]:
        return pulumi.get(self, "nodes")

    @property
    @pulumi.getter
    def password(self) -> Optional[str]:
        return pulumi.get(self, "password")

    @property
    @pulumi.getter
    def username(self) -> Optional[str]:
        return pulumi.get(self, "username")


@pulumi.output_type
class SshNode(dict):
    def __init__(__self__, *,
                 address: str,
                 name: str,
                 port: Optional[int] = None):
        SshNode._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            address=address,
            name=name,
            port=port,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             address: str,
             name: str,
             port: Optional[int] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):

        _setter("address", address)
        _setter("name", name)
        if port is not None:
            _setter("port", port)

    @property
    @pulumi.getter
    def address(self) -> str:
        return pulumi.get(self, "address")

    @property
    @pulumi.getter
    def name(self) -> str:
        return pulumi.get(self, "name")

    @property
    @pulumi.getter
    def port(self) -> Optional[int]:
        return pulumi.get(self, "port")





# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = ['RoleArgs', 'Role']

@pulumi.input_type
class RoleArgs:
    def __init__(__self__, *,
                 privileges: pulumi.Input[Sequence[pulumi.Input[str]]],
                 role_id: pulumi.Input[str]):
        """
        The set of arguments for constructing a Role resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] privileges: The role privileges.
        :param pulumi.Input[str] role_id: The role identifier.
        """
        RoleArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            privileges=privileges,
            role_id=role_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             privileges: pulumi.Input[Sequence[pulumi.Input[str]]],
             role_id: pulumi.Input[str],
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'roleId' in kwargs:
            role_id = kwargs['roleId']

        _setter("privileges", privileges)
        _setter("role_id", role_id)

    @property
    @pulumi.getter
    def privileges(self) -> pulumi.Input[Sequence[pulumi.Input[str]]]:
        """
        The role privileges.
        """
        return pulumi.get(self, "privileges")

    @privileges.setter
    def privileges(self, value: pulumi.Input[Sequence[pulumi.Input[str]]]):
        pulumi.set(self, "privileges", value)

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> pulumi.Input[str]:
        """
        The role identifier.
        """
        return pulumi.get(self, "role_id")

    @role_id.setter
    def role_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "role_id", value)


@pulumi.input_type
class _RoleState:
    def __init__(__self__, *,
                 privileges: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 role_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering Role resources.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] privileges: The role privileges.
        :param pulumi.Input[str] role_id: The role identifier.
        """
        _RoleState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            privileges=privileges,
            role_id=role_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             privileges: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
             role_id: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'roleId' in kwargs:
            role_id = kwargs['roleId']

        if privileges is not None:
            _setter("privileges", privileges)
        if role_id is not None:
            _setter("role_id", role_id)

    @property
    @pulumi.getter
    def privileges(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The role privileges.
        """
        return pulumi.get(self, "privileges")

    @privileges.setter
    def privileges(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "privileges", value)

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> Optional[pulumi.Input[str]]:
        """
        The role identifier.
        """
        return pulumi.get(self, "role_id")

    @role_id.setter
    def role_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "role_id", value)


class Role(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 privileges: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 role_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages a role.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        operations_monitoring = proxmoxve.permission.Role("operationsMonitoring",
            privileges=["VM.Monitor"],
            role_id="operations-monitoring")
        ```

        ## Import

        Instances can be imported using the `role_id`, e.g., bash

        ```sh
         $ pulumi import proxmoxve:Permission/role:Role operations_monitoring operations-monitoring
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] privileges: The role privileges.
        :param pulumi.Input[str] role_id: The role identifier.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: RoleArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a role.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        operations_monitoring = proxmoxve.permission.Role("operationsMonitoring",
            privileges=["VM.Monitor"],
            role_id="operations-monitoring")
        ```

        ## Import

        Instances can be imported using the `role_id`, e.g., bash

        ```sh
         $ pulumi import proxmoxve:Permission/role:Role operations_monitoring operations-monitoring
        ```

        :param str resource_name: The name of the resource.
        :param RoleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(RoleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            RoleArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 privileges: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 role_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = RoleArgs.__new__(RoleArgs)

            if privileges is None and not opts.urn:
                raise TypeError("Missing required property 'privileges'")
            __props__.__dict__["privileges"] = privileges
            if role_id is None and not opts.urn:
                raise TypeError("Missing required property 'role_id'")
            __props__.__dict__["role_id"] = role_id
        super(Role, __self__).__init__(
            'proxmoxve:Permission/role:Role',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            privileges: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            role_id: Optional[pulumi.Input[str]] = None) -> 'Role':
        """
        Get an existing Role resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] privileges: The role privileges.
        :param pulumi.Input[str] role_id: The role identifier.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _RoleState.__new__(_RoleState)

        __props__.__dict__["privileges"] = privileges
        __props__.__dict__["role_id"] = role_id
        return Role(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def privileges(self) -> pulumi.Output[Sequence[str]]:
        """
        The role privileges.
        """
        return pulumi.get(self, "privileges")

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> pulumi.Output[str]:
        """
        The role identifier.
        """
        return pulumi.get(self, "role_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = ['CertifiArgs', 'Certifi']

@pulumi.input_type
class CertifiArgs:
    def __init__(__self__, *,
                 certificate: pulumi.Input[str],
                 node_name: pulumi.Input[str],
                 private_key: pulumi.Input[str],
                 certificate_chain: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None):
        """
        The set of arguments for constructing a Certifi resource.
        :param pulumi.Input[str] certificate: The PEM encoded certificate.
        :param pulumi.Input[str] node_name: A node name.
        :param pulumi.Input[str] private_key: The PEM encoded private key.
        :param pulumi.Input[str] certificate_chain: The PEM encoded certificate chain.
        :param pulumi.Input[bool] overwrite: Whether to overwrite an existing certificate
        """
        CertifiArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            certificate=certificate,
            node_name=node_name,
            private_key=private_key,
            certificate_chain=certificate_chain,
            overwrite=overwrite,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             certificate: pulumi.Input[str],
             node_name: pulumi.Input[str],
             private_key: pulumi.Input[str],
             certificate_chain: Optional[pulumi.Input[str]] = None,
             overwrite: Optional[pulumi.Input[bool]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']
        if 'privateKey' in kwargs:
            private_key = kwargs['privateKey']
        if 'certificateChain' in kwargs:
            certificate_chain = kwargs['certificateChain']

        _setter("certificate", certificate)
        _setter("node_name", node_name)
        _setter("private_key", private_key)
        if certificate_chain is not None:
            _setter("certificate_chain", certificate_chain)
        if overwrite is not None:
            _setter("overwrite", overwrite)

    @property
    @pulumi.getter
    def certificate(self) -> pulumi.Input[str]:
        """
        The PEM encoded certificate.
        """
        return pulumi.get(self, "certificate")

    @certificate.setter
    def certificate(self, value: pulumi.Input[str]):
        pulumi.set(self, "certificate", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Input[str]:
        """
        A node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter(name="privateKey")
    def private_key(self) -> pulumi.Input[str]:
        """
        The PEM encoded private key.
        """
        return pulumi.get(self, "private_key")

    @private_key.setter
    def private_key(self, value: pulumi.Input[str]):
        pulumi.set(self, "private_key", value)

    @property
    @pulumi.getter(name="certificateChain")
    def certificate_chain(self) -> Optional[pulumi.Input[str]]:
        """
        The PEM encoded certificate chain.
        """
        return pulumi.get(self, "certificate_chain")

    @certificate_chain.setter
    def certificate_chain(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "certificate_chain", value)

    @property
    @pulumi.getter
    def overwrite(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to overwrite an existing certificate
        """
        return pulumi.get(self, "overwrite")

    @overwrite.setter
    def overwrite(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "overwrite", value)


@pulumi.input_type
class _CertifiState:
    def __init__(__self__, *,
                 certificate: Optional[pulumi.Input[str]] = None,
                 certificate_chain: Optional[pulumi.Input[str]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 file_name: Optional[pulumi.Input[str]] = None,
                 issuer: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 private_key: Optional[pulumi.Input[str]] = None,
                 public_key_size: Optional[pulumi.Input[int]] = None,
                 public_key_type: Optional[pulumi.Input[str]] = None,
                 ssl_fingerprint: Optional[pulumi.Input[str]] = None,
                 start_date: Optional[pulumi.Input[str]] = None,
                 subject: Optional[pulumi.Input[str]] = None,
                 subject_alternative_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None):
        """
        Input properties used for looking up and filtering Certifi resources.
        :param pulumi.Input[str] certificate: The PEM encoded certificate.
        :param pulumi.Input[str] certificate_chain: The PEM encoded certificate chain.
        :param pulumi.Input[str] expiration_date: The expiration date (RFC 3339).
        :param pulumi.Input[str] file_name: The file name.
        :param pulumi.Input[str] issuer: The issuer.
        :param pulumi.Input[str] node_name: A node name.
        :param pulumi.Input[bool] overwrite: Whether to overwrite an existing certificate
        :param pulumi.Input[str] private_key: The PEM encoded private key.
        :param pulumi.Input[int] public_key_size: The public key size.
        :param pulumi.Input[str] public_key_type: The public key type.
        :param pulumi.Input[str] ssl_fingerprint: The SSL fingerprint.
        :param pulumi.Input[str] start_date: The start date (RFC 3339).
        :param pulumi.Input[str] subject: The subject.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] subject_alternative_names: The subject alternative names.
        """
        _CertifiState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            certificate=certificate,
            certificate_chain=certificate_chain,
            expiration_date=expiration_date,
            file_name=file_name,
            issuer=issuer,
            node_name=node_name,
            overwrite=overwrite,
            private_key=private_key,
            public_key_size=public_key_size,
            public_key_type=public_key_type,
            ssl_fingerprint=ssl_fingerprint,
            start_date=start_date,
            subject=subject,
            subject_alternative_names=subject_alternative_names,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             certificate: Optional[pulumi.Input[str]] = None,
             certificate_chain: Optional[pulumi.Input[str]] = None,
             expiration_date: Optional[pulumi.Input[str]] = None,
             file_name: Optional[pulumi.Input[str]] = None,
             issuer: Optional[pulumi.Input[str]] = None,
             node_name: Optional[pulumi.Input[str]] = None,
             overwrite: Optional[pulumi.Input[bool]] = None,
             private_key: Optional[pulumi.Input[str]] = None,
             public_key_size: Optional[pulumi.Input[int]] = None,
             public_key_type: Optional[pulumi.Input[str]] = None,
             ssl_fingerprint: Optional[pulumi.Input[str]] = None,
             start_date: Optional[pulumi.Input[str]] = None,
             subject: Optional[pulumi.Input[str]] = None,
             subject_alternative_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'certificateChain' in kwargs:
            certificate_chain = kwargs['certificateChain']
        if 'expirationDate' in kwargs:
            expiration_date = kwargs['expirationDate']
        if 'fileName' in kwargs:
            file_name = kwargs['fileName']
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']
        if 'privateKey' in kwargs:
            private_key = kwargs['privateKey']
        if 'publicKeySize' in kwargs:
            public_key_size = kwargs['publicKeySize']
        if 'publicKeyType' in kwargs:
            public_key_type = kwargs['publicKeyType']
        if 'sslFingerprint' in kwargs:
            ssl_fingerprint = kwargs['sslFingerprint']
        if 'startDate' in kwargs:
            start_date = kwargs['startDate']
        if 'subjectAlternativeNames' in kwargs:
            subject_alternative_names = kwargs['subjectAlternativeNames']

        if certificate is not None:
            _setter("certificate", certificate)
        if certificate_chain is not None:
            _setter("certificate_chain", certificate_chain)
        if expiration_date is not None:
            _setter("expiration_date", expiration_date)
        if file_name is not None:
            _setter("file_name", file_name)
        if issuer is not None:
            _setter("issuer", issuer)
        if node_name is not None:
            _setter("node_name", node_name)
        if overwrite is not None:
            _setter("overwrite", overwrite)
        if private_key is not None:
            _setter("private_key", private_key)
        if public_key_size is not None:
            _setter("public_key_size", public_key_size)
        if public_key_type is not None:
            _setter("public_key_type", public_key_type)
        if ssl_fingerprint is not None:
            _setter("ssl_fingerprint", ssl_fingerprint)
        if start_date is not None:
            _setter("start_date", start_date)
        if subject is not None:
            _setter("subject", subject)
        if subject_alternative_names is not None:
            _setter("subject_alternative_names", subject_alternative_names)

    @property
    @pulumi.getter
    def certificate(self) -> Optional[pulumi.Input[str]]:
        """
        The PEM encoded certificate.
        """
        return pulumi.get(self, "certificate")

    @certificate.setter
    def certificate(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "certificate", value)

    @property
    @pulumi.getter(name="certificateChain")
    def certificate_chain(self) -> Optional[pulumi.Input[str]]:
        """
        The PEM encoded certificate chain.
        """
        return pulumi.get(self, "certificate_chain")

    @certificate_chain.setter
    def certificate_chain(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "certificate_chain", value)

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> Optional[pulumi.Input[str]]:
        """
        The expiration date (RFC 3339).
        """
        return pulumi.get(self, "expiration_date")

    @expiration_date.setter
    def expiration_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "expiration_date", value)

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> Optional[pulumi.Input[str]]:
        """
        The file name.
        """
        return pulumi.get(self, "file_name")

    @file_name.setter
    def file_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "file_name", value)

    @property
    @pulumi.getter
    def issuer(self) -> Optional[pulumi.Input[str]]:
        """
        The issuer.
        """
        return pulumi.get(self, "issuer")

    @issuer.setter
    def issuer(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "issuer", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[str]]:
        """
        A node name.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter
    def overwrite(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether to overwrite an existing certificate
        """
        return pulumi.get(self, "overwrite")

    @overwrite.setter
    def overwrite(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "overwrite", value)

    @property
    @pulumi.getter(name="privateKey")
    def private_key(self) -> Optional[pulumi.Input[str]]:
        """
        The PEM encoded private key.
        """
        return pulumi.get(self, "private_key")

    @private_key.setter
    def private_key(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "private_key", value)

    @property
    @pulumi.getter(name="publicKeySize")
    def public_key_size(self) -> Optional[pulumi.Input[int]]:
        """
        The public key size.
        """
        return pulumi.get(self, "public_key_size")

    @public_key_size.setter
    def public_key_size(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "public_key_size", value)

    @property
    @pulumi.getter(name="publicKeyType")
    def public_key_type(self) -> Optional[pulumi.Input[str]]:
        """
        The public key type.
        """
        return pulumi.get(self, "public_key_type")

    @public_key_type.setter
    def public_key_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "public_key_type", value)

    @property
    @pulumi.getter(name="sslFingerprint")
    def ssl_fingerprint(self) -> Optional[pulumi.Input[str]]:
        """
        The SSL fingerprint.
        """
        return pulumi.get(self, "ssl_fingerprint")

    @ssl_fingerprint.setter
    def ssl_fingerprint(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ssl_fingerprint", value)

    @property
    @pulumi.getter(name="startDate")
    def start_date(self) -> Optional[pulumi.Input[str]]:
        """
        The start date (RFC 3339).
        """
        return pulumi.get(self, "start_date")

    @start_date.setter
    def start_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "start_date", value)

    @property
    @pulumi.getter
    def subject(self) -> Optional[pulumi.Input[str]]:
        """
        The subject.
        """
        return pulumi.get(self, "subject")

    @subject.setter
    def subject(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "subject", value)

    @property
    @pulumi.getter(name="subjectAlternativeNames")
    def subject_alternative_names(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The subject alternative names.
        """
        return pulumi.get(self, "subject_alternative_names")

    @subject_alternative_names.setter
    def subject_alternative_names(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "subject_alternative_names", value)


class Certifi(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 certificate: Optional[pulumi.Input[str]] = None,
                 certificate_chain: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 private_key: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages the custom SSL/TLS certificate for a specific node.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve
        import pulumi_tls as tls

        proxmox_virtual_environment_certificate_private_key = tls.PrivateKey("proxmoxVirtualEnvironmentCertificatePrivateKey",
            algorithm="RSA",
            rsa_bits=2048)
        proxmox_virtual_environment_certificate_self_signed_cert = tls.SelfSignedCert("proxmoxVirtualEnvironmentCertificateSelfSignedCert",
            key_algorithm=proxmox_virtual_environment_certificate_private_key.algorithm,
            private_key_pem=proxmox_virtual_environment_certificate_private_key.private_key_pem,
            subject=tls.SelfSignedCertSubjectArgs(
                common_name="example.com",
                organization="Terraform Provider for Proxmox",
            ),
            validity_period_hours=8760,
            allowed_uses=[
                "key_encipherment",
                "digital_signature",
                "server_auth",
            ])
        example = proxmoxve.Certifi("example",
            certificate=proxmox_virtual_environment_certificate_self_signed_cert.cert_pem,
            node_name="first-node",
            private_key=proxmox_virtual_environment_certificate_private_key.private_key_pem)
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] certificate: The PEM encoded certificate.
        :param pulumi.Input[str] certificate_chain: The PEM encoded certificate chain.
        :param pulumi.Input[str] node_name: A node name.
        :param pulumi.Input[bool] overwrite: Whether to overwrite an existing certificate
        :param pulumi.Input[str] private_key: The PEM encoded private key.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: CertifiArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages the custom SSL/TLS certificate for a specific node.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve
        import pulumi_tls as tls

        proxmox_virtual_environment_certificate_private_key = tls.PrivateKey("proxmoxVirtualEnvironmentCertificatePrivateKey",
            algorithm="RSA",
            rsa_bits=2048)
        proxmox_virtual_environment_certificate_self_signed_cert = tls.SelfSignedCert("proxmoxVirtualEnvironmentCertificateSelfSignedCert",
            key_algorithm=proxmox_virtual_environment_certificate_private_key.algorithm,
            private_key_pem=proxmox_virtual_environment_certificate_private_key.private_key_pem,
            subject=tls.SelfSignedCertSubjectArgs(
                common_name="example.com",
                organization="Terraform Provider for Proxmox",
            ),
            validity_period_hours=8760,
            allowed_uses=[
                "key_encipherment",
                "digital_signature",
                "server_auth",
            ])
        example = proxmoxve.Certifi("example",
            certificate=proxmox_virtual_environment_certificate_self_signed_cert.cert_pem,
            node_name="first-node",
            private_key=proxmox_virtual_environment_certificate_private_key.private_key_pem)
        ```

        :param str resource_name: The name of the resource.
        :param CertifiArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(CertifiArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            CertifiArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 certificate: Optional[pulumi.Input[str]] = None,
                 certificate_chain: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 overwrite: Optional[pulumi.Input[bool]] = None,
                 private_key: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = CertifiArgs.__new__(CertifiArgs)

            if certificate is None and not opts.urn:
                raise TypeError("Missing required property 'certificate'")
            __props__.__dict__["certificate"] = certificate
            __props__.__dict__["certificate_chain"] = certificate_chain
            if node_name is None and not opts.urn:
                raise TypeError("Missing required property 'node_name'")
            __props__.__dict__["node_name"] = node_name
            __props__.__dict__["overwrite"] = overwrite
            if private_key is None and not opts.urn:
                raise TypeError("Missing required property 'private_key'")
            __props__.__dict__["private_key"] = None if private_key is None else pulumi.Output.secret(private_key)
            __props__.__dict__["expiration_date"] = None
            __props__.__dict__["file_name"] = None
            __props__.__dict__["issuer"] = None
            __props__.__dict__["public_key_size"] = None
            __props__.__dict__["public_key_type"] = None
            __props__.__dict__["ssl_fingerprint"] = None
            __props__.__dict__["start_date"] = None
            __props__.__dict__["subject"] = None
            __props__.__dict__["subject_alternative_names"] = None
        secret_opts = pulumi.ResourceOptions(additional_secret_outputs=["privateKey"])
        opts = pulumi.ResourceOptions.merge(opts, secret_opts)
        super(Certifi, __self__).__init__(
            'proxmoxve:index/certifi:Certifi',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            certificate: Optional[pulumi.Input[str]] = None,
            certificate_chain: Optional[pulumi.Input[str]] = None,
            expiration_date: Optional[pulumi.Input[str]] = None,
            file_name: Optional[pulumi.Input[str]] = None,
            issuer: Optional[pulumi.Input[str]] = None,
            node_name: Optional[pulumi.Input[str]] = None,
            overwrite: Optional[pulumi.Input[bool]] = None,
            private_key: Optional[pulumi.Input[str]] = None,
            public_key_size: Optional[pulumi.Input[int]] = None,
            public_key_type: Optional[pulumi.Input[str]] = None,
            ssl_fingerprint: Optional[pulumi.Input[str]] = None,
            start_date: Optional[pulumi.Input[str]] = None,
            subject: Optional[pulumi.Input[str]] = None,
            subject_alternative_names: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None) -> 'Certifi':
        """
        Get an existing Certifi resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] certificate: The PEM encoded certificate.
        :param pulumi.Input[str] certificate_chain: The PEM encoded certificate chain.
        :param pulumi.Input[str] expiration_date: The expiration date (RFC 3339).
        :param pulumi.Input[str] file_name: The file name.
        :param pulumi.Input[str] issuer: The issuer.
        :param pulumi.Input[str] node_name: A node name.
        :param pulumi.Input[bool] overwrite: Whether to overwrite an existing certificate
        :param pulumi.Input[str] private_key: The PEM encoded private key.
        :param pulumi.Input[int] public_key_size: The public key size.
        :param pulumi.Input[str] public_key_type: The public key type.
        :param pulumi.Input[str] ssl_fingerprint: The SSL fingerprint.
        :param pulumi.Input[str] start_date: The start date (RFC 3339).
        :param pulumi.Input[str] subject: The subject.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] subject_alternative_names: The subject alternative names.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _CertifiState.__new__(_CertifiState)

        __props__.__dict__["certificate"] = certificate
        __props__.__dict__["certificate_chain"] = certificate_chain
        __props__.__dict__["expiration_date"] = expiration_date
        __props__.__dict__["file_name"] = file_name
        __props__.__dict__["issuer"] = issuer
        __props__.__dict__["node_name"] = node_name
        __props__.__dict__["overwrite"] = overwrite
        __props__.__dict__["private_key"] = private_key
        __props__.__dict__["public_key_size"] = public_key_size
        __props__.__dict__["public_key_type"] = public_key_type
        __props__.__dict__["ssl_fingerprint"] = ssl_fingerprint
        __props__.__dict__["start_date"] = start_date
        __props__.__dict__["subject"] = subject
        __props__.__dict__["subject_alternative_names"] = subject_alternative_names
        return Certifi(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def certificate(self) -> pulumi.Output[str]:
        """
        The PEM encoded certificate.
        """
        return pulumi.get(self, "certificate")

    @property
    @pulumi.getter(name="certificateChain")
    def certificate_chain(self) -> pulumi.Output[Optional[str]]:
        """
        The PEM encoded certificate chain.
        """
        return pulumi.get(self, "certificate_chain")

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> pulumi.Output[str]:
        """
        The expiration date (RFC 3339).
        """
        return pulumi.get(self, "expiration_date")

    @property
    @pulumi.getter(name="fileName")
    def file_name(self) -> pulumi.Output[str]:
        """
        The file name.
        """
        return pulumi.get(self, "file_name")

    @property
    @pulumi.getter
    def issuer(self) -> pulumi.Output[str]:
        """
        The issuer.
        """
        return pulumi.get(self, "issuer")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Output[str]:
        """
        A node name.
        """
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter
    def overwrite(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether to overwrite an existing certificate
        """
        return pulumi.get(self, "overwrite")

    @property
    @pulumi.getter(name="privateKey")
    def private_key(self) -> pulumi.Output[str]:
        """
        The PEM encoded private key.
        """
        return pulumi.get(self, "private_key")

    @property
    @pulumi.getter(name="publicKeySize")
    def public_key_size(self) -> pulumi.Output[int]:
        """
        The public key size.
        """
        return pulumi.get(self, "public_key_size")

    @property
    @pulumi.getter(name="publicKeyType")
    def public_key_type(self) -> pulumi.Output[str]:
        """
        The public key type.
        """
        return pulumi.get(self, "public_key_type")

    @property
    @pulumi.getter(name="sslFingerprint")
    def ssl_fingerprint(self) -> pulumi.Output[str]:
        """
        The SSL fingerprint.
        """
        return pulumi.get(self, "ssl_fingerprint")

    @property
    @pulumi.getter(name="startDate")
    def start_date(self) -> pulumi.Output[str]:
        """
        The start date (RFC 3339).
        """
        return pulumi.get(self, "start_date")

    @property
    @pulumi.getter
    def subject(self) -> pulumi.Output[str]:
        """
        The subject.
        """
        return pulumi.get(self, "subject")

    @property
    @pulumi.getter(name="subjectAlternativeNames")
    def subject_alternative_names(self) -> pulumi.Output[Sequence[str]]:
        """
        The subject alternative names.
        """
        return pulumi.get(self, "subject_alternative_names")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['FirewallIPSetArgs', 'FirewallIPSet']

@pulumi.input_type
class FirewallIPSetArgs:
    def __init__(__self__, *,
                 cidrs: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallIPSetCidrArgs']]]] = None,
                 comment: Optional[pulumi.Input[str]] = None,
                 container_id: Optional[pulumi.Input[int]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None):
        """
        The set of arguments for constructing a FirewallIPSet resource.
        :param pulumi.Input[Sequence[pulumi.Input['FirewallIPSetCidrArgs']]] cidrs: IP/CIDR block (multiple blocks supported).
        :param pulumi.Input[str] comment: Arbitrary string annotation.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level aliases.
        :param pulumi.Input[str] name: Network/IP specification in CIDR format.
        :param pulumi.Input[str] node_name: Node name. Leave empty for cluster level aliases.
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level aliases.
        """
        FirewallIPSetArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            cidrs=cidrs,
            comment=comment,
            container_id=container_id,
            name=name,
            node_name=node_name,
            vm_id=vm_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             cidrs: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallIPSetCidrArgs']]]] = None,
             comment: Optional[pulumi.Input[str]] = None,
             container_id: Optional[pulumi.Input[int]] = None,
             name: Optional[pulumi.Input[str]] = None,
             node_name: Optional[pulumi.Input[str]] = None,
             vm_id: Optional[pulumi.Input[int]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'containerId' in kwargs:
            container_id = kwargs['containerId']
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']
        if 'vmId' in kwargs:
            vm_id = kwargs['vmId']

        if cidrs is not None:
            _setter("cidrs", cidrs)
        if comment is not None:
            _setter("comment", comment)
        if container_id is not None:
            _setter("container_id", container_id)
        if name is not None:
            _setter("name", name)
        if node_name is not None:
            _setter("node_name", node_name)
        if vm_id is not None:
            _setter("vm_id", vm_id)

    @property
    @pulumi.getter
    def cidrs(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FirewallIPSetCidrArgs']]]]:
        """
        IP/CIDR block (multiple blocks supported).
        """
        return pulumi.get(self, "cidrs")

    @cidrs.setter
    def cidrs(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallIPSetCidrArgs']]]]):
        pulumi.set(self, "cidrs", value)

    @property
    @pulumi.getter
    def comment(self) -> Optional[pulumi.Input[str]]:
        """
        Arbitrary string annotation.
        """
        return pulumi.get(self, "comment")

    @comment.setter
    def comment(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "comment", value)

    @property
    @pulumi.getter(name="containerId")
    def container_id(self) -> Optional[pulumi.Input[int]]:
        """
        Container ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "container_id")

    @container_id.setter
    def container_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "container_id", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        Network/IP specification in CIDR format.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[str]]:
        """
        Node name. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> Optional[pulumi.Input[int]]:
        """
        VM ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "vm_id")

    @vm_id.setter
    def vm_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "vm_id", value)


@pulumi.input_type
class _FirewallIPSetState:
    def __init__(__self__, *,
                 cidrs: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallIPSetCidrArgs']]]] = None,
                 comment: Optional[pulumi.Input[str]] = None,
                 container_id: Optional[pulumi.Input[int]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None):
        """
        Input properties used for looking up and filtering FirewallIPSet resources.
        :param pulumi.Input[Sequence[pulumi.Input['FirewallIPSetCidrArgs']]] cidrs: IP/CIDR block (multiple blocks supported).
        :param pulumi.Input[str] comment: Arbitrary string annotation.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level aliases.
        :param pulumi.Input[str] name: Network/IP specification in CIDR format.
        :param pulumi.Input[str] node_name: Node name. Leave empty for cluster level aliases.
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level aliases.
        """
        _FirewallIPSetState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            cidrs=cidrs,
            comment=comment,
            container_id=container_id,
            name=name,
            node_name=node_name,
            vm_id=vm_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             cidrs: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallIPSetCidrArgs']]]] = None,
             comment: Optional[pulumi.Input[str]] = None,
             container_id: Optional[pulumi.Input[int]] = None,
             name: Optional[pulumi.Input[str]] = None,
             node_name: Optional[pulumi.Input[str]] = None,
             vm_id: Optional[pulumi.Input[int]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'containerId' in kwargs:
            container_id = kwargs['containerId']
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']
        if 'vmId' in kwargs:
            vm_id = kwargs['vmId']

        if cidrs is not None:
            _setter("cidrs", cidrs)
        if comment is not None:
            _setter("comment", comment)
        if container_id is not None:
            _setter("container_id", container_id)
        if name is not None:
            _setter("name", name)
        if node_name is not None:
            _setter("node_name", node_name)
        if vm_id is not None:
            _setter("vm_id", vm_id)

    @property
    @pulumi.getter
    def cidrs(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['FirewallIPSetCidrArgs']]]]:
        """
        IP/CIDR block (multiple blocks supported).
        """
        return pulumi.get(self, "cidrs")

    @cidrs.setter
    def cidrs(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['FirewallIPSetCidrArgs']]]]):
        pulumi.set(self, "cidrs", value)

    @property
    @pulumi.getter
    def comment(self) -> Optional[pulumi.Input[str]]:
        """
        Arbitrary string annotation.
        """
        return pulumi.get(self, "comment")

    @comment.setter
    def comment(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "comment", value)

    @property
    @pulumi.getter(name="containerId")
    def container_id(self) -> Optional[pulumi.Input[int]]:
        """
        Container ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "container_id")

    @container_id.setter
    def container_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "container_id", value)

    @property
    @pulumi.getter
    def name(self) -> Optional[pulumi.Input[str]]:
        """
        Network/IP specification in CIDR format.
        """
        return pulumi.get(self, "name")

    @name.setter
    def name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "name", value)

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[pulumi.Input[str]]:
        """
        Node name. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "node_name")

    @node_name.setter
    def node_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "node_name", value)

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> Optional[pulumi.Input[int]]:
        """
        VM ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "vm_id")

    @vm_id.setter
    def vm_id(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "vm_id", value)


class FirewallIPSet(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cidrs: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FirewallIPSetCidrArgs']]]]] = None,
                 comment: Optional[pulumi.Input[str]] = None,
                 container_id: Optional[pulumi.Input[int]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        """
        An IPSet allows us to group multiple IP addresses, IP subnets and aliases. Aliases can be
        created on the cluster level, on VM / Container level.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        ipset = proxmoxve.network.FirewallIPSet("ipset",
            node_name=proxmox_virtual_environment_vm["example"]["node_name"],
            vm_id=proxmox_virtual_environment_vm["example"]["vm_id"],
            comment="Managed by Terraform",
            cidrs=[
                proxmoxve.network.FirewallIPSetCidrArgs(
                    name="192.168.0.0/23",
                    comment="Local network 1",
                ),
                proxmoxve.network.FirewallIPSetCidrArgs(
                    name="192.168.0.1",
                    comment="Server 1",
                    nomatch=True,
                ),
                proxmoxve.network.FirewallIPSetCidrArgs(
                    name="192.168.2.1",
                    comment="Server 1",
                ),
            ],
            opts=pulumi.ResourceOptions(depends_on=[proxmox_virtual_environment_vm["example"]]))
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FirewallIPSetCidrArgs']]]] cidrs: IP/CIDR block (multiple blocks supported).
        :param pulumi.Input[str] comment: Arbitrary string annotation.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level aliases.
        :param pulumi.Input[str] name: Network/IP specification in CIDR format.
        :param pulumi.Input[str] node_name: Node name. Leave empty for cluster level aliases.
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level aliases.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[FirewallIPSetArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        An IPSet allows us to group multiple IP addresses, IP subnets and aliases. Aliases can be
        created on the cluster level, on VM / Container level.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        ipset = proxmoxve.network.FirewallIPSet("ipset",
            node_name=proxmox_virtual_environment_vm["example"]["node_name"],
            vm_id=proxmox_virtual_environment_vm["example"]["vm_id"],
            comment="Managed by Terraform",
            cidrs=[
                proxmoxve.network.FirewallIPSetCidrArgs(
                    name="192.168.0.0/23",
                    comment="Local network 1",
                ),
                proxmoxve.network.FirewallIPSetCidrArgs(
                    name="192.168.0.1",
                    comment="Server 1",
                    nomatch=True,
                ),
                proxmoxve.network.FirewallIPSetCidrArgs(
                    name="192.168.2.1",
                    comment="Server 1",
                ),
            ],
            opts=pulumi.ResourceOptions(depends_on=[proxmox_virtual_environment_vm["example"]]))
        ```

        :param str resource_name: The name of the resource.
        :param FirewallIPSetArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(FirewallIPSetArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            FirewallIPSetArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 cidrs: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FirewallIPSetCidrArgs']]]]] = None,
                 comment: Optional[pulumi.Input[str]] = None,
                 container_id: Optional[pulumi.Input[int]] = None,
                 name: Optional[pulumi.Input[str]] = None,
                 node_name: Optional[pulumi.Input[str]] = None,
                 vm_id: Optional[pulumi.Input[int]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = FirewallIPSetArgs.__new__(FirewallIPSetArgs)

            __props__.__dict__["cidrs"] = cidrs
            __props__.__dict__["comment"] = comment
            __props__.__dict__["container_id"] = container_id
            __props__.__dict__["name"] = name
            __props__.__dict__["node_name"] = node_name
            __props__.__dict__["vm_id"] = vm_id
        super(FirewallIPSet, __self__).__init__(
            'proxmoxve:Network/firewallIPSet:FirewallIPSet',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            cidrs: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FirewallIPSetCidrArgs']]]]] = None,
            comment: Optional[pulumi.Input[str]] = None,
            container_id: Optional[pulumi.Input[int]] = None,
            name: Optional[pulumi.Input[str]] = None,
            node_name: Optional[pulumi.Input[str]] = None,
            vm_id: Optional[pulumi.Input[int]] = None) -> 'FirewallIPSet':
        """
        Get an existing FirewallIPSet resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['FirewallIPSetCidrArgs']]]] cidrs: IP/CIDR block (multiple blocks supported).
        :param pulumi.Input[str] comment: Arbitrary string annotation.
        :param pulumi.Input[int] container_id: Container ID. Leave empty for cluster level aliases.
        :param pulumi.Input[str] name: Network/IP specification in CIDR format.
        :param pulumi.Input[str] node_name: Node name. Leave empty for cluster level aliases.
        :param pulumi.Input[int] vm_id: VM ID. Leave empty for cluster level aliases.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _FirewallIPSetState.__new__(_FirewallIPSetState)

        __props__.__dict__["cidrs"] = cidrs
        __props__.__dict__["comment"] = comment
        __props__.__dict__["container_id"] = container_id
        __props__.__dict__["name"] = name
        __props__.__dict__["node_name"] = node_name
        __props__.__dict__["vm_id"] = vm_id
        return FirewallIPSet(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def cidrs(self) -> pulumi.Output[Optional[Sequence['outputs.FirewallIPSetCidr']]]:
        """
        IP/CIDR block (multiple blocks supported).
        """
        return pulumi.get(self, "cidrs")

    @property
    @pulumi.getter
    def comment(self) -> pulumi.Output[Optional[str]]:
        """
        Arbitrary string annotation.
        """
        return pulumi.get(self, "comment")

    @property
    @pulumi.getter(name="containerId")
    def container_id(self) -> pulumi.Output[Optional[int]]:
        """
        Container ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "container_id")

    @property
    @pulumi.getter
    def name(self) -> pulumi.Output[str]:
        """
        Network/IP specification in CIDR format.
        """
        return pulumi.get(self, "name")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> pulumi.Output[Optional[str]]:
        """
        Node name. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> pulumi.Output[Optional[int]]:
        """
        VM ID. Leave empty for cluster level aliases.
        """
        return pulumi.get(self, "vm_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities
from . import outputs
from ._inputs import *

__all__ = ['UserArgs', 'User']

@pulumi.input_type
class UserArgs:
    def __init__(__self__, *,
                 user_id: pulumi.Input[str],
                 acls: Optional[pulumi.Input[Sequence[pulumi.Input['UserAclArgs']]]] = None,
                 comment: Optional[pulumi.Input[str]] = None,
                 email: Optional[pulumi.Input[str]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 first_name: Optional[pulumi.Input[str]] = None,
                 groups: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 keys: Optional[pulumi.Input[str]] = None,
                 last_name: Optional[pulumi.Input[str]] = None,
                 password: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a User resource.
        :param pulumi.Input[str] user_id: The user identifier.
        :param pulumi.Input[Sequence[pulumi.Input['UserAclArgs']]] acls: The access control list (multiple blocks supported).
        :param pulumi.Input[str] comment: The user comment.
        :param pulumi.Input[str] email: The user's email address.
        :param pulumi.Input[bool] enabled: Whether the user account is enabled.
        :param pulumi.Input[str] expiration_date: The user account's expiration date (RFC 3339).
        :param pulumi.Input[str] first_name: The user's first name.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] groups: The user's groups.
        :param pulumi.Input[str] keys: The user's keys.
        :param pulumi.Input[str] last_name: The user's last name.
        :param pulumi.Input[str] password: The user's password. Required for PVE or PAM realms.
        """
        UserArgs._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            user_id=user_id,
            acls=acls,
            comment=comment,
            email=email,
            enabled=enabled,
            expiration_date=expiration_date,
            first_name=first_name,
            groups=groups,
            keys=keys,
            last_name=last_name,
            password=password,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             user_id: pulumi.Input[str],
             acls: Optional[pulumi.Input[Sequence[pulumi.Input['UserAclArgs']]]] = None,
             comment: Optional[pulumi.Input[str]] = None,
             email: Optional[pulumi.Input[str]] = None,
             enabled: Optional[pulumi.Input[bool]] = None,
             expiration_date: Optional[pulumi.Input[str]] = None,
             first_name: Optional[pulumi.Input[str]] = None,
             groups: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
             keys: Optional[pulumi.Input[str]] = None,
             last_name: Optional[pulumi.Input[str]] = None,
             password: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'userId' in kwargs:
            user_id = kwargs['userId']
        if 'expirationDate' in kwargs:
            expiration_date = kwargs['expirationDate']
        if 'firstName' in kwargs:
            first_name = kwargs['firstName']
        if 'lastName' in kwargs:
            last_name = kwargs['lastName']

        _setter("user_id", user_id)
        if acls is not None:
            _setter("acls", acls)
        if comment is not None:
            _setter("comment", comment)
        if email is not None:
            _setter("email", email)
        if enabled is not None:
            _setter("enabled", enabled)
        if expiration_date is not None:
            _setter("expiration_date", expiration_date)
        if first_name is not None:
            _setter("first_name", first_name)
        if groups is not None:
            _setter("groups", groups)
        if keys is not None:
            _setter("keys", keys)
        if last_name is not None:
            _setter("last_name", last_name)
        if password is not None:
            _setter("password", password)

    @property
    @pulumi.getter(name="userId")
    def user_id(self) -> pulumi.Input[str]:
        """
        The user identifier.
        """
        return pulumi.get(self, "user_id")

    @user_id.setter
    def user_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "user_id", value)

    @property
    @pulumi.getter
    def acls(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserAclArgs']]]]:
        """
        The access control list (multiple blocks supported).
        """
        return pulumi.get(self, "acls")

    @acls.setter
    def acls(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserAclArgs']]]]):
        pulumi.set(self, "acls", value)

    @property
    @pulumi.getter
    def comment(self) -> Optional[pulumi.Input[str]]:
        """
        The user comment.
        """
        return pulumi.get(self, "comment")

    @comment.setter
    def comment(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "comment", value)

    @property
    @pulumi.getter
    def email(self) -> Optional[pulumi.Input[str]]:
        """
        The user's email address.
        """
        return pulumi.get(self, "email")

    @email.setter
    def email(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "email", value)

    @property
    @pulumi.getter
    def enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether the user account is enabled.
        """
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> Optional[pulumi.Input[str]]:
        """
        The user account's expiration date (RFC 3339).
        """
        return pulumi.get(self, "expiration_date")

    @expiration_date.setter
    def expiration_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "expiration_date", value)

    @property
    @pulumi.getter(name="firstName")
    def first_name(self) -> Optional[pulumi.Input[str]]:
        """
        The user's first name.
        """
        return pulumi.get(self, "first_name")

    @first_name.setter
    def first_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "first_name", value)

    @property
    @pulumi.getter
    def groups(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The user's groups.
        """
        return pulumi.get(self, "groups")

    @groups.setter
    def groups(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "groups", value)

    @property
    @pulumi.getter
    def keys(self) -> Optional[pulumi.Input[str]]:
        """
        The user's keys.
        """
        return pulumi.get(self, "keys")

    @keys.setter
    def keys(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "keys", value)

    @property
    @pulumi.getter(name="lastName")
    def last_name(self) -> Optional[pulumi.Input[str]]:
        """
        The user's last name.
        """
        return pulumi.get(self, "last_name")

    @last_name.setter
    def last_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "last_name", value)

    @property
    @pulumi.getter
    def password(self) -> Optional[pulumi.Input[str]]:
        """
        The user's password. Required for PVE or PAM realms.
        """
        return pulumi.get(self, "password")

    @password.setter
    def password(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "password", value)


@pulumi.input_type
class _UserState:
    def __init__(__self__, *,
                 acls: Optional[pulumi.Input[Sequence[pulumi.Input['UserAclArgs']]]] = None,
                 comment: Optional[pulumi.Input[str]] = None,
                 email: Optional[pulumi.Input[str]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 first_name: Optional[pulumi.Input[str]] = None,
                 groups: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 keys: Optional[pulumi.Input[str]] = None,
                 last_name: Optional[pulumi.Input[str]] = None,
                 password: Optional[pulumi.Input[str]] = None,
                 user_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering User resources.
        :param pulumi.Input[Sequence[pulumi.Input['UserAclArgs']]] acls: The access control list (multiple blocks supported).
        :param pulumi.Input[str] comment: The user comment.
        :param pulumi.Input[str] email: The user's email address.
        :param pulumi.Input[bool] enabled: Whether the user account is enabled.
        :param pulumi.Input[str] expiration_date: The user account's expiration date (RFC 3339).
        :param pulumi.Input[str] first_name: The user's first name.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] groups: The user's groups.
        :param pulumi.Input[str] keys: The user's keys.
        :param pulumi.Input[str] last_name: The user's last name.
        :param pulumi.Input[str] password: The user's password. Required for PVE or PAM realms.
        :param pulumi.Input[str] user_id: The user identifier.
        """
        _UserState._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            acls=acls,
            comment=comment,
            email=email,
            enabled=enabled,
            expiration_date=expiration_date,
            first_name=first_name,
            groups=groups,
            keys=keys,
            last_name=last_name,
            password=password,
            user_id=user_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             acls: Optional[pulumi.Input[Sequence[pulumi.Input['UserAclArgs']]]] = None,
             comment: Optional[pulumi.Input[str]] = None,
             email: Optional[pulumi.Input[str]] = None,
             enabled: Optional[pulumi.Input[bool]] = None,
             expiration_date: Optional[pulumi.Input[str]] = None,
             first_name: Optional[pulumi.Input[str]] = None,
             groups: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
             keys: Optional[pulumi.Input[str]] = None,
             last_name: Optional[pulumi.Input[str]] = None,
             password: Optional[pulumi.Input[str]] = None,
             user_id: Optional[pulumi.Input[str]] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'expirationDate' in kwargs:
            expiration_date = kwargs['expirationDate']
        if 'firstName' in kwargs:
            first_name = kwargs['firstName']
        if 'lastName' in kwargs:
            last_name = kwargs['lastName']
        if 'userId' in kwargs:
            user_id = kwargs['userId']

        if acls is not None:
            _setter("acls", acls)
        if comment is not None:
            _setter("comment", comment)
        if email is not None:
            _setter("email", email)
        if enabled is not None:
            _setter("enabled", enabled)
        if expiration_date is not None:
            _setter("expiration_date", expiration_date)
        if first_name is not None:
            _setter("first_name", first_name)
        if groups is not None:
            _setter("groups", groups)
        if keys is not None:
            _setter("keys", keys)
        if last_name is not None:
            _setter("last_name", last_name)
        if password is not None:
            _setter("password", password)
        if user_id is not None:
            _setter("user_id", user_id)

    @property
    @pulumi.getter
    def acls(self) -> Optional[pulumi.Input[Sequence[pulumi.Input['UserAclArgs']]]]:
        """
        The access control list (multiple blocks supported).
        """
        return pulumi.get(self, "acls")

    @acls.setter
    def acls(self, value: Optional[pulumi.Input[Sequence[pulumi.Input['UserAclArgs']]]]):
        pulumi.set(self, "acls", value)

    @property
    @pulumi.getter
    def comment(self) -> Optional[pulumi.Input[str]]:
        """
        The user comment.
        """
        return pulumi.get(self, "comment")

    @comment.setter
    def comment(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "comment", value)

    @property
    @pulumi.getter
    def email(self) -> Optional[pulumi.Input[str]]:
        """
        The user's email address.
        """
        return pulumi.get(self, "email")

    @email.setter
    def email(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "email", value)

    @property
    @pulumi.getter
    def enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether the user account is enabled.
        """
        return pulumi.get(self, "enabled")

    @enabled.setter
    def enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "enabled", value)

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> Optional[pulumi.Input[str]]:
        """
        The user account's expiration date (RFC 3339).
        """
        return pulumi.get(self, "expiration_date")

    @expiration_date.setter
    def expiration_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "expiration_date", value)

    @property
    @pulumi.getter(name="firstName")
    def first_name(self) -> Optional[pulumi.Input[str]]:
        """
        The user's first name.
        """
        return pulumi.get(self, "first_name")

    @first_name.setter
    def first_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "first_name", value)

    @property
    @pulumi.getter
    def groups(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        The user's groups.
        """
        return pulumi.get(self, "groups")

    @groups.setter
    def groups(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "groups", value)

    @property
    @pulumi.getter
    def keys(self) -> Optional[pulumi.Input[str]]:
        """
        The user's keys.
        """
        return pulumi.get(self, "keys")

    @keys.setter
    def keys(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "keys", value)

    @property
    @pulumi.getter(name="lastName")
    def last_name(self) -> Optional[pulumi.Input[str]]:
        """
        The user's last name.
        """
        return pulumi.get(self, "last_name")

    @last_name.setter
    def last_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "last_name", value)

    @property
    @pulumi.getter
    def password(self) -> Optional[pulumi.Input[str]]:
        """
        The user's password. Required for PVE or PAM realms.
        """
        return pulumi.get(self, "password")

    @password.setter
    def password(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "password", value)

    @property
    @pulumi.getter(name="userId")
    def user_id(self) -> Optional[pulumi.Input[str]]:
        """
        The user identifier.
        """
        return pulumi.get(self, "user_id")

    @user_id.setter
    def user_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "user_id", value)


class User(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 acls: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['UserAclArgs']]]]] = None,
                 comment: Optional[pulumi.Input[str]] = None,
                 email: Optional[pulumi.Input[str]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 first_name: Optional[pulumi.Input[str]] = None,
                 groups: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 keys: Optional[pulumi.Input[str]] = None,
                 last_name: Optional[pulumi.Input[str]] = None,
                 password: Optional[pulumi.Input[str]] = None,
                 user_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages a user.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        operations_monitoring = proxmoxve.permission.Role("operationsMonitoring",
            role_id="operations-monitoring",
            privileges=["VM.Monitor"])
        operations_automation = proxmoxve.permission.User("operationsAutomation",
            acls=[proxmoxve.permission.UserAclArgs(
                path="/vms/1234",
                propagate=True,
                role_id=operations_monitoring.role_id,
            )],
            comment="Managed by Terraform",
            password="a-strong-password",
            user_id="operations-automation@pve")
        ```

        ## Import

        Instances can be imported using the `user_id`, e.g., bash

        ```sh
         $ pulumi import proxmoxve:Permission/user:User operations_automation operations-automation@pve
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['UserAclArgs']]]] acls: The access control list (multiple blocks supported).
        :param pulumi.Input[str] comment: The user comment.
        :param pulumi.Input[str] email: The user's email address.
        :param pulumi.Input[bool] enabled: Whether the user account is enabled.
        :param pulumi.Input[str] expiration_date: The user account's expiration date (RFC 3339).
        :param pulumi.Input[str] first_name: The user's first name.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] groups: The user's groups.
        :param pulumi.Input[str] keys: The user's keys.
        :param pulumi.Input[str] last_name: The user's last name.
        :param pulumi.Input[str] password: The user's password. Required for PVE or PAM realms.
        :param pulumi.Input[str] user_id: The user identifier.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: UserArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a user.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_proxmoxve as proxmoxve

        operations_monitoring = proxmoxve.permission.Role("operationsMonitoring",
            role_id="operations-monitoring",
            privileges=["VM.Monitor"])
        operations_automation = proxmoxve.permission.User("operationsAutomation",
            acls=[proxmoxve.permission.UserAclArgs(
                path="/vms/1234",
                propagate=True,
                role_id=operations_monitoring.role_id,
            )],
            comment="Managed by Terraform",
            password="a-strong-password",
            user_id="operations-automation@pve")
        ```

        ## Import

        Instances can be imported using the `user_id`, e.g., bash

        ```sh
         $ pulumi import proxmoxve:Permission/user:User operations_automation operations-automation@pve
        ```

        :param str resource_name: The name of the resource.
        :param UserArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(UserArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            kwargs = kwargs or {}
            def _setter(key, value):
                kwargs[key] = value
            UserArgs._configure(_setter, **kwargs)
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 acls: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['UserAclArgs']]]]] = None,
                 comment: Optional[pulumi.Input[str]] = None,
                 email: Optional[pulumi.Input[str]] = None,
                 enabled: Optional[pulumi.Input[bool]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 first_name: Optional[pulumi.Input[str]] = None,
                 groups: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 keys: Optional[pulumi.Input[str]] = None,
                 last_name: Optional[pulumi.Input[str]] = None,
                 password: Optional[pulumi.Input[str]] = None,
                 user_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = UserArgs.__new__(UserArgs)

            __props__.__dict__["acls"] = acls
            __props__.__dict__["comment"] = comment
            __props__.__dict__["email"] = email
            __props__.__dict__["enabled"] = enabled
            __props__.__dict__["expiration_date"] = expiration_date
            __props__.__dict__["first_name"] = first_name
            __props__.__dict__["groups"] = groups
            __props__.__dict__["keys"] = keys
            __props__.__dict__["last_name"] = last_name
            __props__.__dict__["password"] = password
            if user_id is None and not opts.urn:
                raise TypeError("Missing required property 'user_id'")
            __props__.__dict__["user_id"] = user_id
        super(User, __self__).__init__(
            'proxmoxve:Permission/user:User',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            acls: Optional[pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['UserAclArgs']]]]] = None,
            comment: Optional[pulumi.Input[str]] = None,
            email: Optional[pulumi.Input[str]] = None,
            enabled: Optional[pulumi.Input[bool]] = None,
            expiration_date: Optional[pulumi.Input[str]] = None,
            first_name: Optional[pulumi.Input[str]] = None,
            groups: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            keys: Optional[pulumi.Input[str]] = None,
            last_name: Optional[pulumi.Input[str]] = None,
            password: Optional[pulumi.Input[str]] = None,
            user_id: Optional[pulumi.Input[str]] = None) -> 'User':
        """
        Get an existing User resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[Sequence[pulumi.Input[pulumi.InputType['UserAclArgs']]]] acls: The access control list (multiple blocks supported).
        :param pulumi.Input[str] comment: The user comment.
        :param pulumi.Input[str] email: The user's email address.
        :param pulumi.Input[bool] enabled: Whether the user account is enabled.
        :param pulumi.Input[str] expiration_date: The user account's expiration date (RFC 3339).
        :param pulumi.Input[str] first_name: The user's first name.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] groups: The user's groups.
        :param pulumi.Input[str] keys: The user's keys.
        :param pulumi.Input[str] last_name: The user's last name.
        :param pulumi.Input[str] password: The user's password. Required for PVE or PAM realms.
        :param pulumi.Input[str] user_id: The user identifier.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _UserState.__new__(_UserState)

        __props__.__dict__["acls"] = acls
        __props__.__dict__["comment"] = comment
        __props__.__dict__["email"] = email
        __props__.__dict__["enabled"] = enabled
        __props__.__dict__["expiration_date"] = expiration_date
        __props__.__dict__["first_name"] = first_name
        __props__.__dict__["groups"] = groups
        __props__.__dict__["keys"] = keys
        __props__.__dict__["last_name"] = last_name
        __props__.__dict__["password"] = password
        __props__.__dict__["user_id"] = user_id
        return User(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter
    def acls(self) -> pulumi.Output[Optional[Sequence['outputs.UserAcl']]]:
        """
        The access control list (multiple blocks supported).
        """
        return pulumi.get(self, "acls")

    @property
    @pulumi.getter
    def comment(self) -> pulumi.Output[Optional[str]]:
        """
        The user comment.
        """
        return pulumi.get(self, "comment")

    @property
    @pulumi.getter
    def email(self) -> pulumi.Output[Optional[str]]:
        """
        The user's email address.
        """
        return pulumi.get(self, "email")

    @property
    @pulumi.getter
    def enabled(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether the user account is enabled.
        """
        return pulumi.get(self, "enabled")

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> pulumi.Output[Optional[str]]:
        """
        The user account's expiration date (RFC 3339).
        """
        return pulumi.get(self, "expiration_date")

    @property
    @pulumi.getter(name="firstName")
    def first_name(self) -> pulumi.Output[Optional[str]]:
        """
        The user's first name.
        """
        return pulumi.get(self, "first_name")

    @property
    @pulumi.getter
    def groups(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        The user's groups.
        """
        return pulumi.get(self, "groups")

    @property
    @pulumi.getter
    def keys(self) -> pulumi.Output[Optional[str]]:
        """
        The user's keys.
        """
        return pulumi.get(self, "keys")

    @property
    @pulumi.getter(name="lastName")
    def last_name(self) -> pulumi.Output[Optional[str]]:
        """
        The user's last name.
        """
        return pulumi.get(self, "last_name")

    @property
    @pulumi.getter
    def password(self) -> pulumi.Output[Optional[str]]:
        """
        The user's password. Required for PVE or PAM realms.
        """
        return pulumi.get(self, "password")

    @property
    @pulumi.getter(name="userId")
    def user_id(self) -> pulumi.Output[str]:
        """
        The user identifier.
        """
        return pulumi.get(self, "user_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Callable, Mapping, Optional, Sequence, Union, overload
from .. import _utilities

__all__ = [
    'GroupAcl',
    'PoolMember',
    'UserAcl',
    'GetGroupAclResult',
    'GetPoolMemberResult',
    'GetUserAclResult',
]

@pulumi.output_type
class GroupAcl(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "roleId":
            suggest = "role_id"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in GroupAcl. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        GroupAcl.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        GroupAcl.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 path: str,
                 role_id: str,
                 propagate: Optional[bool] = None):
        """
        :param str path: The path.
        :param str role_id: The role identifier.
        :param bool propagate: Whether to propagate to child paths.
        """
        GroupAcl._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            path=path,
            role_id=role_id,
            propagate=propagate,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             path: str,
             role_id: str,
             propagate: Optional[bool] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'roleId' in kwargs:
            role_id = kwargs['roleId']

        _setter("path", path)
        _setter("role_id", role_id)
        if propagate is not None:
            _setter("propagate", propagate)

    @property
    @pulumi.getter
    def path(self) -> str:
        """
        The path.
        """
        return pulumi.get(self, "path")

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> str:
        """
        The role identifier.
        """
        return pulumi.get(self, "role_id")

    @property
    @pulumi.getter
    def propagate(self) -> Optional[bool]:
        """
        Whether to propagate to child paths.
        """
        return pulumi.get(self, "propagate")


@pulumi.output_type
class PoolMember(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "datastoreId":
            suggest = "datastore_id"
        elif key == "nodeName":
            suggest = "node_name"
        elif key == "vmId":
            suggest = "vm_id"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in PoolMember. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        PoolMember.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        PoolMember.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 datastore_id: Optional[str] = None,
                 id: Optional[str] = None,
                 node_name: Optional[str] = None,
                 type: Optional[str] = None,
                 vm_id: Optional[int] = None):
        """
        :param str datastore_id: The datastore identifier.
        :param str id: The member identifier.
        :param str node_name: The node name.
        :param str type: The member type.
        :param int vm_id: The virtual machine identifier.
        """
        PoolMember._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            datastore_id=datastore_id,
            id=id,
            node_name=node_name,
            type=type,
            vm_id=vm_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             datastore_id: Optional[str] = None,
             id: Optional[str] = None,
             node_name: Optional[str] = None,
             type: Optional[str] = None,
             vm_id: Optional[int] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'datastoreId' in kwargs:
            datastore_id = kwargs['datastoreId']
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']
        if 'vmId' in kwargs:
            vm_id = kwargs['vmId']

        if datastore_id is not None:
            _setter("datastore_id", datastore_id)
        if id is not None:
            _setter("id", id)
        if node_name is not None:
            _setter("node_name", node_name)
        if type is not None:
            _setter("type", type)
        if vm_id is not None:
            _setter("vm_id", vm_id)

    @property
    @pulumi.getter(name="datastoreId")
    def datastore_id(self) -> Optional[str]:
        """
        The datastore identifier.
        """
        return pulumi.get(self, "datastore_id")

    @property
    @pulumi.getter
    def id(self) -> Optional[str]:
        """
        The member identifier.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> Optional[str]:
        """
        The node name.
        """
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter
    def type(self) -> Optional[str]:
        """
        The member type.
        """
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> Optional[int]:
        """
        The virtual machine identifier.
        """
        return pulumi.get(self, "vm_id")


@pulumi.output_type
class UserAcl(dict):
    @staticmethod
    def __key_warning(key: str):
        suggest = None
        if key == "roleId":
            suggest = "role_id"

        if suggest:
            pulumi.log.warn(f"Key '{key}' not found in UserAcl. Access the value via the '{suggest}' property getter instead.")

    def __getitem__(self, key: str) -> Any:
        UserAcl.__key_warning(key)
        return super().__getitem__(key)

    def get(self, key: str, default = None) -> Any:
        UserAcl.__key_warning(key)
        return super().get(key, default)

    def __init__(__self__, *,
                 path: str,
                 role_id: str,
                 propagate: Optional[bool] = None):
        """
        :param str path: The path.
        :param str role_id: The role identifier.
        :param bool propagate: Whether to propagate to child paths.
        """
        UserAcl._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            path=path,
            role_id=role_id,
            propagate=propagate,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             path: str,
             role_id: str,
             propagate: Optional[bool] = None,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'roleId' in kwargs:
            role_id = kwargs['roleId']

        _setter("path", path)
        _setter("role_id", role_id)
        if propagate is not None:
            _setter("propagate", propagate)

    @property
    @pulumi.getter
    def path(self) -> str:
        """
        The path.
        """
        return pulumi.get(self, "path")

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> str:
        """
        The role identifier.
        """
        return pulumi.get(self, "role_id")

    @property
    @pulumi.getter
    def propagate(self) -> Optional[bool]:
        """
        Whether to propagate to child paths.
        """
        return pulumi.get(self, "propagate")


@pulumi.output_type
class GetGroupAclResult(dict):
    def __init__(__self__, *,
                 path: str,
                 propagate: bool,
                 role_id: str):
        """
        :param str path: The path.
        :param bool propagate: Whether to propagate to child paths.
        :param str role_id: The role identifier.
        """
        GetGroupAclResult._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            path=path,
            propagate=propagate,
            role_id=role_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             path: str,
             propagate: bool,
             role_id: str,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'roleId' in kwargs:
            role_id = kwargs['roleId']

        _setter("path", path)
        _setter("propagate", propagate)
        _setter("role_id", role_id)

    @property
    @pulumi.getter
    def path(self) -> str:
        """
        The path.
        """
        return pulumi.get(self, "path")

    @property
    @pulumi.getter
    def propagate(self) -> bool:
        """
        Whether to propagate to child paths.
        """
        return pulumi.get(self, "propagate")

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> str:
        """
        The role identifier.
        """
        return pulumi.get(self, "role_id")


@pulumi.output_type
class GetPoolMemberResult(dict):
    def __init__(__self__, *,
                 datastore_id: str,
                 id: str,
                 node_name: str,
                 type: str,
                 vm_id: int):
        """
        :param str datastore_id: The datastore identifier.
        :param str id: The member identifier.
        :param str node_name: The node name.
        :param str type: The member type.
        :param int vm_id: The virtual machine identifier.
        """
        GetPoolMemberResult._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            datastore_id=datastore_id,
            id=id,
            node_name=node_name,
            type=type,
            vm_id=vm_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             datastore_id: str,
             id: str,
             node_name: str,
             type: str,
             vm_id: int,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'datastoreId' in kwargs:
            datastore_id = kwargs['datastoreId']
        if 'nodeName' in kwargs:
            node_name = kwargs['nodeName']
        if 'vmId' in kwargs:
            vm_id = kwargs['vmId']

        _setter("datastore_id", datastore_id)
        _setter("id", id)
        _setter("node_name", node_name)
        _setter("type", type)
        _setter("vm_id", vm_id)

    @property
    @pulumi.getter(name="datastoreId")
    def datastore_id(self) -> str:
        """
        The datastore identifier.
        """
        return pulumi.get(self, "datastore_id")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The member identifier.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="nodeName")
    def node_name(self) -> str:
        """
        The node name.
        """
        return pulumi.get(self, "node_name")

    @property
    @pulumi.getter
    def type(self) -> str:
        """
        The member type.
        """
        return pulumi.get(self, "type")

    @property
    @pulumi.getter(name="vmId")
    def vm_id(self) -> int:
        """
        The virtual machine identifier.
        """
        return pulumi.get(self, "vm_id")


@pulumi.output_type
class GetUserAclResult(dict):
    def __init__(__self__, *,
                 path: str,
                 propagate: bool,
                 role_id: str):
        """
        :param str path: The path.
        :param bool propagate: Whether to propagate to child paths.
        :param str role_id: The role identifier.
        """
        GetUserAclResult._configure(
            lambda key, value: pulumi.set(__self__, key, value),
            path=path,
            propagate=propagate,
            role_id=role_id,
        )
    @staticmethod
    def _configure(
             _setter: Callable[[Any, Any], None],
             path: str,
             propagate: bool,
             role_id: str,
             opts: Optional[pulumi.ResourceOptions]=None,
             **kwargs):
        if 'roleId' in kwargs:
            role_id = kwargs['roleId']

        _setter("path", path)
        _setter("propagate", propagate)
        _setter("role_id", role_id)

    @property
    @pulumi.getter
    def path(self) -> str:
        """
        The path.
        """
        return pulumi.get(self, "path")

    @property
    @pulumi.getter
    def propagate(self) -> bool:
        """
        Whether to propagate to child paths.
        """
        return pulumi.get(self, "propagate")

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> str:
        """
        The role identifier.
        """
        return pulumi.get(self, "role_id")


